@@ -22,7 +22,7 @@ try:
     from scipy.interpolate import griddata
 except:
     sys.exit("Please install scipy")
-    
+
 import utils
 try:
     from fort.DFT import dft
@@ -30,13 +30,13 @@ try:
 except:
     print "Warning: DFT fortran routines not found"
     fort_routines = False
-    
+
 
 #==============================================================================
 # Functions
 #==============================================================================
 
-def power_spec_1d_fast(quantity, bins=None, min_wave=0.0001, 
+def power_spec_1d_fast(quantity, bins=None, min_wave=0.0001,
                        max_wave=0.02, pad=1, weight=None):
     """
     Computes the 1D power spectrum and plots it.
@@ -76,23 +76,32 @@ def power_spec_1d_fast(quantity, bins=None, min_wave=0.0001,
     # First bin the quantity (higher number of bins tends towards phase calculation)
     if not bins:
         bins = utils.FD_bins(quantity)
-
-    if weight:
-        hist, edges = np.histogram(quantity, bins, 
+        print "bins: ", bins
+    if weight is not None:
+        hist, edges = np.histogram(quantity, bins,
                                    weights=weight)
+        print np.min(weight)
+        print np.max(weight)
     else:
         hist, edges = np.histogram(quantity, bins)
 
+    print np.min(quantity)
+    print np.max(quantity)
+    print "hist: ", hist
+    print "edges: ", edges
     #Calculate stepsize.
     step = edges[2] - edges[1]
-     
+
     n = pad * hist.size
     ps = np.abs(np.fft.rfft(hist, n=n) / np.sqrt(n)) ** 2
-    
+
     freq = np.fft.fftfreq(n, step)
 
     wavelength = 1.0 / freq[:np.floor(n / 2 + 1)]
 
+    print "ps: ", ps
+    print "freq ", freq
+    print "wvl ", wavelength
     return ps, freq * 2.0 * np.pi, wavelength
 
 
@@ -117,41 +126,41 @@ def power_spec_1d_slow(quantity, bins=None, min_wave=0.0001, max_wave=0.02, n_wa
     """
     #Get an equally-spaced array of wavelengths
     wavelengths = np.asfortranarray(np.linspace(min_wave, max_wave, n_waves))
-    freq = 2*np.pi / wavelengths
+    freq = 2 * np.pi / wavelengths
     #If a number of bins is given then bin it up and do it faster, if not, do "phase" calc
     if bins:
-        if weight:
-            hist, edges = np.histogram(quantity, bins, 
+        if weight is not None:
+            hist, edges = np.histogram(quantity, bins,
                                        weights=weight)
         else:
             hist, edges = np.histogram(quantity, bins)
 
-        centres = np.asfortranarray(edges[:-1] + (edges[1]-edges[0])/2.0)
+        centres = np.asfortranarray(edges[:-1] + (edges[1] - edges[0]) / 2.0)
         ps = dft.dft_one(np.asfortranarray(hist), centres, wavelengths)
-    
+
     else:
-        if not weight:
+        if weight is None:
             ps = dft.phasedft_one(np.asfortranarray(quantity), wavelengths)
         else:
-            ps = dft.dft_one(np.asfortranarray(weight), 
-                             np.asfortranarray(quantity), 
+            ps = dft.dft_one(np.asfortranarray(weight),
+                             np.asfortranarray(quantity),
                              wavelengths)
 
-    return ps, freq,wavelengths
+    return ps, freq, wavelengths
 
 #    def PowerSpec_2d_fast(self, quantity=("c_dist", "ra"), bins=None, min_wave=(0.0001, 0.0001), max_wave=(1000.0, 1000.0),
 #                     pad=(1, 1), wavelength=True):
 #        """
 #        Returns and plots the 2d power spectra of the given quantities
-#        
+#
 #        This is the equivalent procedure to PowerSpec_1d_fast for 2-dimensions. However, in this case,
 #        the plotting becomes more difficult. The default plots are density images, with colours showing
-#        peaks. The irregular nature of the wavelength plots is pronounced in 2D however. 
-#        
+#        peaks. The irregular nature of the wavelength plots is pronounced in 2D however.
+#
 #        To produce the wavelength plots, a regular grid is created and the original power spectrum
-#        is interpolated onto it. This produces very striated plots however. To get nice plots in 
+#        is interpolated onto it. This produces very striated plots however. To get nice plots in
 #        wavelength, use PowerSpec_2d_slow().
-#        
+#
 #        INPUT PARAMETERS
 #        quantity (tuple) [('c_dist','ra')]    :the quantities to use on (x,y) axes.
 #        bins (int)        [None]              : the number of bins to use in both directions
@@ -159,14 +168,14 @@ def power_spec_1d_slow(quantity, bins=None, min_wave=0.0001, max_wave=0.02, n_wa
 #        max_wave (tuple)    [(1000.0,1000.0)] :the maximum wavelengths to use.
 #        pad      (tuple)    [(1,1)]            :the factor use for padding with zeroes.
 #        wavelength (bool)    [True]            :whether to plot against wavelength
-#        
+#
 #        OUTPUT PARAMETERS
 #        cps    : the 2D power spectrum in the range given
 #        cfreq1 : the frequencies in the first quantity
 #        cfreq2 : the frequencies in the second quantity
 #        wvl1   : the wavelengths in the first quantity
 #        wvl2   : the wavlengths in the second quantity
-#        
+#
 #        """
 #
 #
@@ -256,15 +265,15 @@ def power_spec_1d_slow(quantity, bins=None, min_wave=0.0001, max_wave=0.02, n_wa
 #    def PowerSpec_2d_slow(self, quantity=("c_dist", "ra"), bins=None, min_wave=(0.0001, 0.0001), max_wave=(1000.0, 1000.0), n_waves=(1000, 1000)):
 #        """
 #        Returns and plots the 2d power spectra of the given quantities
-#        
+#
 #        This is the equivalent procedure to PowerSpec_1d_fast for 2-dimensions. However, in this case,
 #        the plotting becomes more difficult. The default plots are density images, with colours showing
-#        peaks. The irregular nature of the wavelength plots is pronounced in 2D however. 
-#        
+#        peaks. The irregular nature of the wavelength plots is pronounced in 2D however.
+#
 #        To produce the wavelength plots, a regular grid is created and the original power spectrum
-#        is interpolated onto it. This produces very striated plots however. To get nice plots in 
+#        is interpolated onto it. This produces very striated plots however. To get nice plots in
 #        wavelength, use PowerSpec_2d_slow().
-#        
+#
 #        INPUT PARAMETERS
 #        quantity (tuple) [('c_dist','ra')]    :the quantities to use on (x,y) axes.
 #        bins (int)        [None]              : the number of bins to use in both directions, if none, do phase.
@@ -272,12 +281,12 @@ def power_spec_1d_slow(quantity, bins=None, min_wave=0.0001, max_wave=0.02, n_wa
 #        max_wave (tuple)    [(1000.0,1000.0)] :the maximum wavelengths to use.
 #        n_waves  (tuple)    [(1000,1000)]     : the number of wavlengths to use.
 #        wavelength (bool)    [True]            :whether to plot against wavelength
-#        
+#
 #        OUTPUT PARAMETERS
 #        ps    : the 2D power spectrum in the range given
 #        wvl1   : the wavelengths in the first quantity
 #        wvl2   : the wavlengths in the second quantity
-#        
+#
 #        """
 #
 #        wvl1 = np.asfortranarray(np.linspace(min_wave[0], max_wave[0], n_waves[0]))
@@ -318,17 +327,17 @@ def power_spec_1d_slow(quantity, bins=None, min_wave=0.0001, max_wave=0.02, n_wa
 #                          max_wave=(1000.0, 1000.0, 1000.0), pad=(1, 1, 1), wavelength=False):
 #        """
 #        Returns and plots the 2d power spectra of the given quantities
-#        
+#
 #        At the moment this does not work correctly. This will be fixed soon.
-#        
+#
 #        This is the equivalent procedure to PowerSpec_1d_fast for 2-dimensions. However, in this case,
 #        the plotting becomes more difficult. The default plots are density images, with colours showing
-#        peaks. The irregular nature of the wavelength plots is pronounced in 2D however. 
-#        
+#        peaks. The irregular nature of the wavelength plots is pronounced in 2D however.
+#
 #        To produce the wavelength plots, a regular grid is created and the original power spectrum
-#        is interpolated onto it. This produces very striated plots however. To get nice plots in 
+#        is interpolated onto it. This produces very striated plots however. To get nice plots in
 #        wavelength, use PowerSpec_2d_slow().
-#        
+#
 #        INPUT PARAMETERS
 #        quantity (tuple) [('c_dist','ra')]    :the quantities to use on (x,y) axes.
 #        bins (int)        [None]              : the number of bins to use in both directions
@@ -336,14 +345,14 @@ def power_spec_1d_slow(quantity, bins=None, min_wave=0.0001, max_wave=0.02, n_wa
 #        max_wave (tuple)    [(1000.0,1000.0)] :the maximum wavelengths to use.
 #        pad      (tuple)    [(1,1)]            :the factor use for padding with zeroes.
 #        wavelength (bool)    [True]            :whether to plot against wavelength
-#        
+#
 #        OUTPUT PARAMETERS
 #        cps    : the 2D power spectrum in the range given
 #        cfreq1 : the frequencies in the first quantity
 #        cfreq2 : the frequencies in the second quantity
 #        wvl1   : the wavelengths in the first quantity
 #        wvl2   : the wavlengths in the second quantity
-#        
+#
 #        """
 #
 #
@@ -401,21 +410,21 @@ def power_spec_1d_slow(quantity, bins=None, min_wave=0.0001, max_wave=0.02, n_wa
 #                          max_wave=(1000.0, 1000.0, 1000.0), n_waves=(1000, 1000, 1000)):
 #        """
 #        Returns and plots the 2d power spectra of the given quantities
-#        
+#
 #        This is the equivalent procedure to PowerSpec_1d_fast for 2-dimensions. However, in this case,
 #        the plotting becomes more difficult. The default plots are density images, with colours showing
-#        peaks. The irregular nature of the wavelength plots is pronounced in 2D however. 
-#        
+#        peaks. The irregular nature of the wavelength plots is pronounced in 2D however.
+#
 #        To produce the wavelength plots, a regular grid is created and the original power spectrum
-#        is interpolated onto it. This produces very striated plots however. To get nice plots in 
+#        is interpolated onto it. This produces very striated plots however. To get nice plots in
 #        wavelength, use PowerSpec_2d_slow().
-#        
+#
 #        INPUT PARAMETERS
 #        quantity (tuple) [('c_dist','ra')]    :the quantities to use on (x,y) axes.
 #        bins (int)        [None]              : the number of bins to use in both directions, if none, do phase.
 #        min_wave (tuple)    [(0.0001,0.0001)] : the minimum wavelengths to use
-#    
-#            
+#
+#
 #        """
 #
 #        wvl1 = np.asfortranarray(np.linspace(min_wave[0], max_wave[0], n_waves[0]))
