@@ -24,6 +24,15 @@ Created on May 22, 2012
 '''
 history = \
 """
+0.5.0 - 06/11/2013
+        MANY MANY CHANGES - I haven't been updating the history for most of them
+        Mostly now there is working luminosity functions and selection functions
+        Non-interactive Import is still a bit buggy.
+        
+0.4.2 - 02/05/2013
+        Minor updates and bugfixes
+        Convert pandas series to numpy array for FD_bins function.
+        
 0.4.1 - 30/05/2013
         Re-organised code so that external modules have intrinsic functions which
         are USED by PyGS but not automatically classes - makes it easier to trace
@@ -121,12 +130,16 @@ try:
 except:
     sys.exit("Please install Numpy")
 
+try:
+    import matplotlib.pyplot as plt
+except:
+    sys.exit("Please install matplotlib")
 
 #Modules in PyGS
 import LuminosityFunction as lf
 #from SimplePlots import SimplePlots as sp
 import PowerSpectra as ps
-#from Correlations import Correlations as corr
+import Correlations as cf
 import utils
 import SelectionFunction as sf
 #from Cosmology import Cosmology as csm
@@ -143,7 +156,7 @@ class Import(object):
                  sample_name=None,  #Which sample to use (can always be 'all', otherwise will make one)
                  sample_cuts=None,  #Cuts to apply to the sample
                  clobber=None,  #Whether to automatically overwrite previous project folders
-                 verbose = True):
+                 verbose=True):
         '''
         Imports the survey and creates a project directory and internal structure. 
         
@@ -202,6 +215,7 @@ class Import(object):
             # Whether to overwrite project directory if appropriate
                 
         '''
+        #TODO: doesn't work if you choose NOT to overwrite if inputting .csv (line 640 survey_data doesn't exist!)
 
         #Save the cosmology (default fiducial cosmology of cosmolopy).
         self.cosmology = cosmology
@@ -211,8 +225,8 @@ class Import(object):
         self.verbose = verbose
         self.plottype = 'png'
 
-        #The columns wanted from the original file
-        self._raw_columns = ["redshift", "ra", "dec", "mag_r", "mag_g", "id"]
+        #The columns wanted from the original file (some of these are not needed after read-in though)
+        self._raw_columns = ["redshift", "ra", "dec", "mag_r", "mag_g", "id", 'extinction_r', "extinction_g"]
 
         #Determine whether the given filename is a project file or not - also check whether there is a project file for the survey.
         #filename is returned modified if a project file was found for the input filename (which wasn't a project file)
@@ -232,38 +246,38 @@ class Import(object):
 
 
         # Now we definitely have a HDF5 project file so we carry on.
-        # Define the sample
+        # Define the sample: get the sample name, the cuts and turn these into a query object
         self.sample_name, self.sample_cuts, query = self._define_sample(filename)
+        print query
+        #Create the sample directory (if it doesn't already exist)
+        self._create_sample_dir(self.sample_name)
+        #Save the sample directory path to the instance
+        self._sample_dir = os.path.join(self._project_dir, self.sample_name)
+        #Create results directories inside the sample
+        self._make_result_dirs()
 
         #Write the sample cuts and cosmo to file for next time
-        self._write_sample_cuts_cosmo(filename,self.sample_name,self.sample_cuts)
-        
+        self._write_sample_cuts_cosmo(filename, self.sample_name, self.sample_cuts)
+
         #Import the project file
         self._project_file_import(filename, query)
 
+        #Apply sample cuts
+        self._cut_data(self.sample_cuts)
+
         # Now the base data is all in. We need to calculate extra properties.
 
         #Calculate the comoving distance to each particle.
         self.survey_data['c_dist'] = cd.comoving_distance(self.survey_data["redshift"], **self.cosmology)
-        lum_dist = lf.lum_dist(self.survey_data['redshift'],self.survey_data['c_dist'])
+        lum_dist = lf.lum_dist(self.survey_data['redshift'], self.survey_data['c_dist'])
 
-        #Here we have three options. Either mag_r AND mag_g are in the data, or only mag_r, or only mag_g or none.
-        #We can only do K-Correction if both apparent magnitudes are present.
-        if 'mag_r' in self.survey_data.columns and 'mag_g' in self.survey_data.columns:
-            #Find the g-r colour and K-Correction to find Abs_Mag and luminosity.
-            gr = self.survey_data["mag_g"] - self.survey_data["mag_r"]
-            self.survey_data['kcorr'] = lf.k_correct(self.survey_data['redshift'],gr)
 
-            self.survey_data["absmag"] = lf.absolute_mag(self.survey_data['mag_r'],lum_dist, self.survey_data['kcorr'])
-            self.survey_data['lum'] = lf.absmag_to_lum(self.survey_data['absmag'])
-            
-        elif 'mag_g' in self.survey_data.dtype.names:
-            #calculate absmag and luminosities without K-Correction
-            self.survey_data["absmag"] = lf.absolute_mag(self.survey_data['mag_r'],lum_dist)
-            self.survey_data['lum'] = lf.absmag_to_lum(self.survey_data['absmag'])
+        #Find the g-r colour and K-Correction to find Abs_Mag and luminosity.
+        gr = self.survey_data["mag_g"] - self.survey_data["mag_r"] - self.survey_data["extinction_g"] - self.survey_data["extinction_r"]
+        self.survey_data['kcorr'] = lf.k_correct(self.survey_data['redshift'], gr)
 
-        else:
-            sys.exit("You have no apparent magnitude in g-band specified. Try again.")
+        self.survey_data["absmag"] = lf.absolute_mag(self.survey_data['mag_r'], lum_dist, self.survey_data['kcorr'], self.survey_data["extinction_r"])
+        self.survey_data['lum'] = lf.absmag_to_lum(self.survey_data['absmag'])
 
         #Delete stupid entries
         self.clean()
@@ -272,7 +286,7 @@ class Import(object):
         self.N = self.survey_data.shape[0]
         self.survey_name = os.path.split(self._project_dir)[1].replace('_Project', '')
 
-        self._make_result_dirs()
+
         self._define_z_from_dist()
 
     def _determine_if_project(self, filename):
@@ -284,9 +298,13 @@ class Import(object):
         #If the folder containing the file contains _PyLSSProject_ it is a project file
         if "_PyLSSProject_" in os.listdir(os.path.split(filename)[0]):
             is_project_file = True
+            project_dir = os.path.split(filename)[0]
         elif os.path.isdir(project_dir):
             if self._clobber is None:
-                do_continue = raw_input("A project folder for this survey has already been created, would you like to overwrite it (y) or use its contents (n)?")
+                do_continue = raw_input("A project folder for this survey has " +
+                                        "already been created, would you like " +
+                                        "to overwrite it (y) or use its " +
+                                        "contents (n)?")
                 while do_continue[0] not in 'yn':
                     do_continue = raw_input("Overwrite? Please type y or n: ")
 
@@ -328,7 +346,7 @@ class Import(object):
         """
 
         plot_dirs = ['SurveyPlots', 'PowerSpecs_1D', 'PowerSpecs_2D', 'FilteredData',
-                     'CorrelationFunctions', 'LuminosityFunction']
+                     'CorrelationFunctions', 'LuminosityFunction', 'SelectionFunction']
         self._dirs = {}
         for folder in plot_dirs:
             self._dirs[folder] = self._sample_dir + '/' + folder + '/'
@@ -341,19 +359,19 @@ class Import(object):
         # If the filename is actually a file, just read it in.
         if os.path.isfile(filename):
             self.survey_data = pandas.read_csv(filename)
-
-        #Test whether the filename is either of the above
         else:
             sys.exit("Your filename was not a true filename. Filename: " + filename)
 
+        #We rename all columns in lower case for simplicity
         for col in self.survey_data.columns:
             self.survey_data.rename(columns={col:col.lower()}, inplace=True)
 
+
+        print "This survey contains ", len(self.survey_data), " galaxies"
         # Print out the fields read in.
         print "The field names of the data just read are: "
         print self.survey_data.columns
 
-
         #Check if all the raw columns are already there, then the job is done.
         got_all = True
         for col in self._raw_columns:
@@ -362,11 +380,10 @@ class Import(object):
             self.survey_data = self.survey_data[self._raw_columns]
             return
 
-
         print "Attempting automatic conversion to PyGS format... ",
         aliases = {'redshift':['z'],
-                   'mag_r':['appmag_r', 'app_mag_r', 'fibermag_r', 'modelmag_r'],
-                   'mag_g':['appmag_g', 'app_mag_g', 'fibermag_g', 'modelmag_g'],
+                   'mag_r':['modelmag_r', 'cmodelmag_r'],
+                   'mag_g':['modelmag_g', 'cmodelmag_g'],
                    'id':['specobjid', 'specobj_id', 'objid']}
 
         for key, val in aliases.iteritems():
@@ -382,9 +399,9 @@ class Import(object):
             print "succeeded."
         elif self.col_mapping:
             print "failed."
-            self.survey_data.rename(columns=self.col_mapping,inplace=True)
+            self.survey_data.rename(columns=self.col_mapping, inplace=True)
         else:
-            
+
             print "failed. "
             print "Please input column names manually - Modified (current) field names are: "
             print self.survey_data.columns
@@ -405,10 +422,13 @@ class Import(object):
 
         #Just a uniqueness test
         if 'id' in self.survey_data.columns:
-            print "Testing for Uniqueness"
+            print "Testing for Uniqueness:"
             unique, indices = np.unique(np.array(self.survey_data['id']), return_index=True)
+            print "Deleting ", len(self.survey_data) - len(unique), " duplicate values"
             self.survey_data = self.survey_data.iloc[indices]
-            del(unique, indices, self.survey_data['id'])
+            del unique
+            del indices
+            del self.survey_data['id']
 
         #Cull all the columns we don't care about
         restriction = []
@@ -471,39 +491,58 @@ class Import(object):
 
         store.close()
 
-    def _write_sample_cuts_cosmo(self,filename,sample_name,sample_cuts):
-        
+    def _write_sample_cuts_cosmo(self, filename, sample_name, sample_cuts):
+
         store = pandas.HDFStore(filename)
-        
-        #Write the sample_cuts ('all') to project file
-        store[sample_name+'_cuts'] = sample_cuts
+
+        #Write the sample_cuts  to project file
+        store[sample_name + '_cuts'] = sample_cuts
 
         #Write the cosmology to project file
         cosmo = pandas.DataFrame(index=[0])
         for key, val in self.cosmology.iteritems():
             cosmo[key] = [val]
-        store[sample_name+'_cosmo'] = cosmo
+        store[sample_name + '_cosmo'] = cosmo
+
 
-        self._create_sample_dir('all')
-        
     def _project_file_import(self, filename, query):
         """
         A simple function to import the project data
         """
         store = pandas.HDFStore(filename)
-        if query:
+        if query is not None:
             self.survey_data = store.select('survey', query)
+            print "Using query"
         else:
             self.survey_data = store['survey']
 
+    def _cut_data(self, sample_cuts):
+        """
+        Cuts the data with the provided sample cuts.. only here because store.select() doesn't work??
+        """
+
+        for key, val in sample_cuts.iteritems():
+            if val[0] is not None:
+                self.survey_data = self.survey_data[self.survey_data[key] > val[0]]
+                if len(self.survey_data) == 0:
+                    raise Exception("Cuts left data with no galaxies!")
+            if val[1] is not None:
+                self.survey_data = self.survey_data[self.survey_data[key] < val[1]]
+                if len(self.survey_data) == 0:
+                    raise Exception("Cuts left data with no galaxies!")
+
+        return
+
+
     def _define_sample(self, filename):
         """
         A convenience method which will let the user choose a pre-made sample or make one themselves
         """
         store = pandas.HDFStore(filename)
-
+        print os.getcwd()
+        print self._project_dir
         sample_names = os.walk(self._project_dir).next()[1]
-        
+
         if not self.sample_name:
             print "Current defined samples are: "
             for name in sample_names:
@@ -537,8 +576,6 @@ class Import(object):
         #Define query string from sample cuts
         query = self._make_query(sample_cuts)
 
-        self._create_sample_dir(sample_name)
-
         return sample_name, sample_cuts, query
 
 
@@ -558,34 +595,35 @@ class Import(object):
 
     def _convert_sample_cuts(self):
         """ Converts sample cuts in form of dict to writable form"""
-        
+
         sample_cuts = pandas.DataFrame(index=["min", "max"])
         for col in self._raw_columns:
             try:
                 sample_cuts[col] = self.sample_cuts[col]
             except:
-                sample_cuts[col] = [None,None]
-                
+                sample_cuts[col] = [None, None]
+
         return sample_cuts
-        
+
     def _create_sample_dir(self, sample_name):
         """
         Creates a directory for the sample
         """
-        self._sample_dir = os.path.join(self._project_dir, sample_name)
+        sample_dir = os.path.join(self._project_dir, sample_name)
 
         try:
-            os.makedirs(self._sample_dir)
+            os.makedirs(sample_dir)
         except OSError, e:
             if e.errno != errno.EEXIST:
                 raise
 
-        os.chdir(self._sample_dir)
+        os.chdir(sample_dir)
 
         # Create the sample file identifier
         sample_file = open("_PyLSSSample_", "w")
         sample_file.close()
 
+        return sample_dir
     def _specify_sample_cuts(self):
         """
         Specifies sample cuts and makes those cuts to current data
@@ -602,7 +640,7 @@ class Import(object):
                 except SyntaxError:
                     min_input = None
                     print ""
-    
+
                 try:
                     max_input = input("What is the maximum " + col + " preferred? ["
                                                       + str(min(self.survey_data[col])) + ','
@@ -610,8 +648,8 @@ class Import(object):
                 except SyntaxError:
                     max_input = None
                     print ""
-    
-                sample_cuts[col] = [min_input,max_input]
+
+                sample_cuts[col] = [min_input, max_input]
         return sample_cuts
 
 
@@ -629,45 +667,117 @@ class Import(object):
         """
 
         #Delete entries that have absmag > 0 (evidently not correct)
-        print "Deleting ", np.sum(self.survey_data['absmag'] > 0), " entries"
+        print "Deleting ", np.sum(self.survey_data['absmag'] > 0), " entries whose absmag is greater than 0"
         self.survey_data = self.survey_data[self.survey_data['absmag'] < 0]
 
         #Delete entries that have absmag < -30 (evidently not correct)
-        print "Deleting ", np.sum(self.survey_data['absmag'] < -30), " entries"
+        print "Deleting ", np.sum(self.survey_data['absmag'] < -30), " entries whose absmag is less than -30"
         self.survey_data = self.survey_data[self.survey_data['absmag'] > -30]
 
 
+#===============================================================================
+# Some methods that people can actually access
+#===============================================================================
+    def write_individual_files(self, prefix="", exclude=[]):
+        """
+        Write out an ascii file for each column in the survey_data to prefix
+        
+        e.g. if prefix = "/home/user/" then you'll get the files /home/user/z.dat,
+        /home/user/absmag.dat etc.
+        
+        Not specifying prefix by default writes the files in the current directory
+        (which should be the sample directory).
+        """
+        for col in self.survey_data.columns:
+            if col not in exclude:
+                self.survey_data[col].to_csv(os.path.join(prefix, col + '.dat'), index=False, header=False)
 
 
 #==============================================================================
 #   Now we start individual "modules" within the class
 #==============================================================================
-    
+
 # --------------- Luminosity Funtion methods ----------------------------------
-    def lum_func(Np,min_acceptable=0,eps=0.01):
+    def lum_func(self, Np, method="vmax", min_acceptable=1, eps=0.01,
+                 max_iter=10, new=True):
         """
         Calculates the luminosity function of galaxies in Np bins.
-        Uses EEP method only currently (that may be expanded)
+        
+        Can use either the EEP method or the Vmax method currently.
+        
+        INPUT PARAMETERS
+        Np                 -- The number of bins to use
+        method ["vmax"]    -- A string indicating which method to use (EEP or vmax)
+        min_acceptable [1] -- Minimum acceptable galaxies in a bin
+        eps [0.01]         -- for EEP, target convergence
+        max_iter [10]      -- for EEP, maximum iterations
+        new [True]         -- whether to force re-calculation even if previous results are available
+        
+        OUTPUT
+        M   -- Array of absolute magnitude bin centres
+        phi -- Array of luminosity function values
         """
-        
-        phi, iteration = lf.lum_func(absmag = self.survey_data['absmag'],
-                                     mag_r = self.survey_data['mag_r'],
-                                     c_dist = self.survey_data['c_dist'],
-                                     k_corr = self.survey_data['k_corr'],
-                                     Np = Np,
-                                     min_acceptable=minacceptable,
-                                     eps=eps,
-                                     verbose=self.verbose
-                                     )
-        self.LF = phi[iteration]
-        self.LF_iterations = phi
-        
-        
+
+        #We first look to find a pre-calculated function if wanted
+        if not new:
+            try:
+                lfunc = np.genfromtxt(self._dirs["LuminosityFunction"] + method + ".dat")
+                M = lfunc[:, 0]
+                phi = lfunc[:, 1]
+                return M, phi
+            except IOError:
+                pass
+
+        if method == "EEP":
+            phi, iteration = lf.EEP(absmag=self.survey_data['absmag'],
+                                    apmag=self.survey_data['mag_r'],
+                                    Np=Np,
+                                    min_acceptable=min_acceptable,
+                                    eps=eps,
+                                    max_iter=max_iter,
+                                    verbose=self.verbose)
+
+            #Plot all iterations
+            phi.plot(logy=True, marker='o')
+            plt.show()
+            plt.savefig(self._dirs["LuminosityFunction"] + "allEEP." + self.plottype)
+
+            M = np.array(phi.index)
+            phi = np.array(phi[iteration])
+
+            #Plot the final answer
+            plt.clf()
+            plt.plot(M, phi)
+            plt.xlabel("Absolute Magnitude")
+            plt.ylabel("Unnormalised Luminosity Function")
+            plt.yscale('log')
+            plt.savefig(self._dirs["LuminosityFunction"] + "finalEEP." + self.plottype)
+
+
+        elif method == "vmax":
+            phi, M = lf.vmax_method(absmag=self.survey_data['absmag'],
+                                 appmag=self.survey_data['mag_r'],
+                                 c_dist=self.survey_data["c_dist"],
+                                 redshift=self.survey_data['redshift'],
+                                 bins=Np)
+
+            #Plot it up
+            plt.clf()
+            plt.plot(M, phi)
+            plt.yscale('log')
+            plt.savefig(self._dirs["LuminosityFunction"] + "vmax." + self.plottype)
+
+        #Save results to file
+        output = np.vstack((M, phi)).T
+        np.savetxt(self._dirs["LuminosityFunction"] + method + ".dat", output)
+
+        return M, phi
+
 # -------------- PowerSpectra methods -----------------------------------------
-        
-    def power_spec(quantity="redshift", bins=None, min_wave=0.0001, 
-                      max_wave=0.02, n_waves = 1000,pad=1, in_wavelength=True, 
-                      weight=True,slow=True):
+
+    def power_spec(self, quantity="redshift", bins=None, min_wave=0.0001,
+                      max_wave=0.02, n_waves=1000, pad=1, in_wavelength=True,
+                      weight=True, slow=True, yscale='linear', xscale='linear'):
         """
         Computes the 1D power spectrum and plots it.
     
@@ -702,7 +812,7 @@ class Import(object):
         freq                      : array of frequencies
         wvl                       : array of wavelengths
         """
-        
+
         #Set some default options
         if quantity is 'c_dist' and max_wave < 2:
             min_wave = 10
@@ -713,37 +823,38 @@ class Import(object):
         elif quantity is 'dec' and min_wave == 0.0001 and max_wave == 0.02:
             min_wave = 0.01
             max_wave = np.pi
-            
+
         if quantity != 'redshift' and quantity != 'c_dist':
             weight = False
-            
+
         #Get weights if not already in the data
         if weight:
             if 'weight' not in self.survey_data.columns:
                 self.do_selection_and_weighting()
             weight = self.survey_data['weight']
-            
+        else:
+            weight = None
         #Do calculations
         if not slow:
-            power, freq, wavelength = ps.power_spec_1d_fast(quantity = self.survey_data['quantity'],
-                                                  bins=bins, 
+            power, freq, wavelength = ps.power_spec_1d_fast(quantity=self.survey_data[quantity],
+                                                  bins=bins,
                                                   min_wave=min_wave,
-                                                  max_wave=max_wave, 
-                                                  pad=pad,  
+                                                  max_wave=max_wave,
+                                                  pad=pad,
                                                   weight=weight)
         else:
-            power, freq, wavelength = ps.power_spec_1d_slow(quantity = self.survey_data['quantity'],
-                                                  bins=bins, 
+            power, freq, wavelength = ps.power_spec_1d_slow(quantity=self.survey_data[quantity],
+                                                  bins=bins,
                                                   min_wave=min_wave,
-                                                  max_wave=max_wave, 
-                                                  n_waves = n_waves, 
+                                                  max_wave=max_wave,
+                                                  n_waves=n_waves,
                                                   weight=weight)
-        #Plot the function                                                  
+        #Plot the function
         plt.clf()
         plt.title("1D P.S. for " + quantity + " of sample " + self.sample_name.partition('.')[0] + \
                   " from " + self.survey_name)
         if in_wavelength:
-            plt.plot(wavelength[(wavelength < max_wave) & (wavelength > min_wave)], 
+            plt.plot(wavelength[(wavelength < max_wave) & (wavelength > min_wave)],
                                 power[(wavelength < max_wave) & (wavelength > min_wave)], 'b-')
             if quantity == 'redshift':
                 plt.xlabel("Wavelength, (z)")
@@ -754,50 +865,188 @@ class Import(object):
                           power[(wavelength < max_wave) & (wavelength > min_wave)], 'b-')
             plt.xlabel("Frequency (radians)")
         plt.ylabel("Power")
+        plt.xscale(xscale);plt.yscale(yscale)
         if in_wavelength:
             plt.savefig(self._dirs['PowerSpecs_1D'] + quantity + '_bins' + str(bins) + '_wavelength.eps')
         else:
             plt.savefig(self._dirs['PowerSpecs_1D'] + quantity + '_bins' + str(bins) + '_frequency.eps')
-        
-        plt.show()
-        
-#==============================================================================
-#  Selection Function Routines 
-#==============================================================================
-        
-    def do_selection_and_weighting(self,bins=None):
-        """
-        Performs selection function fitting based on simple selection form only
-        """
-        
-        A,g,t = sf.fit_selection_simple(self.survey_data['redshift'],
-                                     bins=bins,verbose=self.verbose)
-        
-        self.survey_data['weight'] = 1 / sf.selection_simple(self.survey_data['redshift'], 
-                                                             A=A,g=g,t=t)
-                                                            
-        self.selection_params = {'A':A,'g':g,'t':t}
 
-    def create_radial_selection(self,N=None):
+        plt.show()
+
+#==============================================================================
+#  Selection Function Routines
+#==============================================================================
+
+    def selection_function(self, method="EEP", bins=None,
+                           new_lumfunc=False, min_acceptable=1, eps=0.01,
+                           max_iter=10, selbins=100):
+        """
+        Calculate a selection function value for each galaxy.
+        
+        INPUT PARAMETERS
+        method ["EEP"]        -- A method with which to calculate the lumfunc. If None,
+                                 will use a simple parametrisation for the N(z) distribution.                    
+        bins [None]           -- How many bins to use in the given method
+        new_lumfunc [False]   -- Whether to force a new lumfunc calculation even if previous
+                                 results exist
+        new_selection [False] -- Whether to force a new selection function calcualtion even
+                                 if previous results exit.
+        min_acceptable        -- Lowest acceptable number of galaxies per bin
+        eps                   -- Target uncertainty for convergence in EEP
+        max_iter              -- Maximum iterations for EEP
+                            
+        """
+        if method is None:
+            A, g, t = sf.fit_selection_simple(self.survey_data['c_dist'],
+                                         bins=bins, verbose=self.verbose)
+
+            self.survey_data['weight'] = 1.0 / sf.get_sel_from_nd(A, g, t)(self.survey_data['c_dist'])
+
+            #WRITE OUT A SIMPLE PLOT FOR CHECKING!
+            if not bins:
+                bins = utils.FD_bins(self.survey_data['c_dist'])
+            plt.clf()
+            plt.title("N(d) with selection function for Sample " + self.sample_name + " from " + self.survey_name)
+            hist, edges = np.histogram(self.survey_data['c_dist'], density=True, bins=bins)
+            centres = edges[:-1] + (edges[1] - edges[0]) / 2.0
+            plt.plot(centres, hist)
+            plt.plot(centres, sf.selection_simple(centres, A, g, t))
+            plt.xlabel("Comoving Distance")
+            plt.ylabel("Normalised Count")
+            plt.savefig(self._dirs['SelectionFunction'] + 'N(d)_' + str(bins) + "bins.eps")
+
+
+            plt.show()
+
+            self.selection_params = {'A':A, 'g':g, 't':t}
+
+        else:
+            M, phi = self.lum_func(Np=bins, method=method,
+                                       new=new_lumfunc, min_acceptable=min_acceptable,
+                                       eps=eps, max_iter=max_iter)
+            if selbins is None:
+                self.survey_data['weight'] = 1. / sf.selection_of_galaxies(phi, M,
+                                                                           self.survey_data['mag_r'],
+                                                                           self.survey_data["absmag"])
+
+                #Plot it up
+                plt.clf()
+                plt.plot(self.survey_data["redshift"], 1. / self.survey_data["weight"],
+                         linestyle="None", marker='.', markersize=1)
+                plt.xlabel("Redshift")
+                plt.ylabel("Selection Function")
+                plt.yscale("log")
+                plt.savefig(self._dirs['SelectionFunction'] + 'SelectionFunction_' + method + '_' + str(bins) + '.' + self.plottype)
+            else:
+                s_of_z = sf.selection_of_z(selbins, phi, M, self.survey_data["mag_r"],
+                                           self.survey_data["absmag"],
+                                           self.survey_data["redshift"])
+                self.survey_data['weight'] = 1. / np.exp(s_of_z(self.survey_data['redshift']))
+
+                #Make a plot
+                z = np.linspace(self.survey_data['redshift'].min(),
+                                self.survey_data['redshift'].max(),
+                                1000)
+                plt.clf()
+                plt.plot(z, np.exp(s_of_z(z)))
+                plt.yscale('log')
+                plt.xlabel("Redshift")
+                plt.ylabel("Selection Function")
+
+                plt.savefig(self._dirs['SelectionFunction'] +
+                            'SelectionFunctionAve_' + method + '_L' + str(bins)
+                            + "_S" + str(selbins) + '.' + self.plottype)
+
+    def radial_mock(self, method="EEP", N=None, bins=100,
+                    new_lumfunc=False, min_acceptable=1, eps=0.01,
+                    max_iter=10, selbins=100):
         """
         Creates a random catalogue of particles based on a defined selection function. Only radial co-ords.
         
         Ouput: radii :: length N vector (default length of survey) of radii        
         """
 
-        z_min = self.survey_data['redshift'].min
-        z_max = self.survey_data['redshift'].max           
+        z_min = self.survey_data['redshift'].min()
+        z_max = self.survey_data['redshift'].max()
 
-        #Generate some random numbers
         if N is None:
-            N = self.N                                         
-            
-        radii = sf.create_radial_selection(z_min,z_max, N)
-        
+            N = self.N
+
+        if method is None:
+            radii = sf.create_radial_selection_simple(z_min, z_max, N)
+
+        else:
+            M, phi = self.lum_func(Np=bins, method=method,
+                                   new=new_lumfunc, min_acceptable=min_acceptable,
+                                   eps=eps, max_iter=max_iter)
+            s_of_z = sf.selection_of_z(selbins, phi, M, self.survey_data["mag_r"],
+                                       self.survey_data["absmag"],
+                                       self.survey_data["redshift"])
+            radii = sf.create_mock(s_of_z, z_min, z_max, N)
+
+        #Plot it
+        plt.clf()
+        plt.hist(radii, bins=100, normed=True, label="Mock")
+        plt.hist(self.survey_data['redshift'], bins=100, normed=True, label="Data")
+        plt.legend()
+        plt.savefig(self._dirs['SelectionFunction'] +
+                    'N(z)_' + method + '_L' + str(bins)
+                    + "_S" + str(selbins) + '.' + self.plottype)
         return radii
-        
+
+#===============================================================================
+# Correlation Function Methods
+#===============================================================================
+    def corr_one_d(self, quantity='redshift', end_s=0.02, ns=15, n_randoms=1, n_cores=1):
+        """
+        Calculates the correlation function in 1d for the given quantity
+        """
+
+        if quantity == 'c_dist' and end_s == 0.02:
+            end_s = 120
+
+        if quantity == 'redshift':
+            min_cut = self.sample_cuts['redshift'][0]
+            max_cut = self.sample_cuts['redshift'][1]
+        elif quantity == 'c_dist':
+            min_cut = cd.comoving_distance(self.sample_cuts['redshift'][0], **self.cosmology)
+            max_cut = cd.comoving_distance(self.sample_cuts['redshift'][1], **self.cosmology)
+        else:
+            raise ValueError("Quantity not supported: ", quantity)
+
+        s_bins, corr = cf._1d_correlation(self.survey_data[quantity], min_cut, max_cut, end_s, ns, n_randoms, n_cores)
+
+        print s_bins, corr
+
+        plt.clf()
+        plt.plot(s_bins, corr)
+        plt.title(r"Correlation Function for $" + quantity + "$ of sample " + self.sample_name.partition('.')[0] + \
+                  " from " + self.survey_name)
+        print "made title"
+        if quantity == 'c_dist':
+            plt.xlabel(r'Separation, Mpc $h^{-1}$s')
+        elif quantity == "redshift":
+            plt.xlabel(r'Separation, [z]')
+        print "plotted"
+        plt.ylabel(r"$\xi(r)$")
+        plt.savefig(self._dirs["CorrelationFunctions"] + "CorrelationFunction_" + quantity + '_NS' + str(ns) + '_Nrand' + str(n_randoms) + '.eps')
+
+        print "saved first fig"
+        plt.clf()
+        plt.plot(s_bins, corr * s_bins ** 2)
+        plt.title(r"Correlation Function by $s^2$ for $" + quantity + "$ of sample " + self.sample_name.partition('.')[0] + \
+                  " from " + self.survey_name)
+        if quantity == 'c_dist':
+            plt.xlabel(r'Separation, Mpc $h^{-1}$s')
+        elif quantity == "redshift":
+            plt.xlabel(r'Separation, [z]')
+
+        plt.ylabel(r"$r^2\xi(r)$")
+        plt.savefig(self._dirs["CorrelationFunctions"] + "CorrelationFunction_s2_" + quantity + '_NS' + str(ns) + '_Nrand' + str(n_randoms) + '.eps')
+
+        return s_bins, corr
 #==============================================================================
-# Simple Plots 
+# Simple Plots
 #==============================================================================
 
     def PlotHist(self, quantity="redshift", bins=None, weight=None):
@@ -814,7 +1063,7 @@ class Import(object):
         if not weight:
             plt.hist(self.survey_data[quantity], bins)
         else:
-            plt.hist(self.survey_data[quantity], bins, weights=self.survey_data[weight])
+            plt.hist(self.survey_data[quantity], bins, weights=self.survey_data['weight'])
 
         plt.xlabel(quantity)
         plt.ylabel("Number")
